program pinger;
{$APPTYPE CONSOLE}
uses
  classes,
  Windows,
  SysUtils,
  IdIcmpClient,
  IdBaseComponent,
  IdComponent,
  IdRawBase,
  IdRawClient,
  DOS in 'DOS.pas';

Const VER = '20140211.13';
{
 2014-02-10 Created


 Build requirements: Delphi 5, Indy 9
}

type TPinger = class(TObject)
  ICMP: TIdIcmpClient;
  procedure ICMPReply(ASender: TComponent; const ReplyStatus: TReplyStatus);
private
  constructor Create(sHost: string);
public
end;

var
  sLogFileName: string;
  LogFile: TextFile;
  oPinger: TPinger;
  sHost: string;
  nInterval: single;
  nTimeout: single;
  nMaxFail: integer;
  sFailCmd: string;
  nPings: Int64;
  nFails: Int64;
  nConsecutiveFails: Integer;

procedure Log(s: string);
begin
  s := FormatDateTime('yyyy-mm-dd hh:nn:ss',Now) + ' - ' + s;
  WriteLn (s);
  Append(LogFile);
  WriteLn(LogFile,s);
  CloseFile(LogFile)
end;

procedure TPinger.ICMPReply(ASender: TComponent; const ReplyStatus: TReplyStatus);
  var s: string;
begin
  inc(nPings);
  s := IntToStr(ReplyStatus.MsRoundTripTime);
  if ReplyStatus.BytesReceived = 0 then
  begin
    s := s+' (FAIL)';
    inc(nFails);
    inc(nConsecutiveFails);
  end else
  begin
    nConsecutiveFails := 0;
  end;
  Log (s);
end;

constructor TPinger.Create(sHost: string);
begin
  inherited Create;
  ICMP := TIdIcmpClient.Create(nil);
  ICMP.Host := sHost;
  ICMP.ReceiveTimeout := round(nTimeout*1000);
end;

procedure usage;
begin
  WriteLn;
  WriteLn('Pinger, version '+VER+' by Charles Boling');
  WriteLn;
  WriteLn('Continually pings a host until halted with Ctrl-C.');
  WriteLn('Timestamped log will appear in '+ExtractFilePath(ParamStr(0)));
  WriteLn;
  WriteLn('USAGE:   '+ExtractFileName(ParamStr(0))+' host [option=value] [option=value] ... ');
  WriteLn('         (Host = Name or IP address of device to ping');
  WriteLn('');
  WriteLn('EXAMPLE: pinger test.com interval=1 timeout=.08 "FailCmd=page 5551212 Slow Net!"');
  WriteLn('');
  WriteLn('OPTIONS: ');
  WriteLn('Interval = number of seconds between pings (decimals okay); default=5');
  WriteLn('Timeout  = seconds to wait for response (decimals okay); default=1');
  WriteLn('MaxFail  = Max # of consecutive failures before alert; default=5');
  WriteLn('FailCmd  = Command to execute after MaxFail failures; default=""');
  WriteLn('           (Also executes at 5*MaxFail, 20*MaxFail)');
  WriteLn('           Surround entire parameter in quotes if command has spaces, etc.');
  WriteLn('');
  WriteLn('Host must be the first parameter; order of other options doesn''t matter.');
  WriteLn;
  WriteLn;
  Halt(ParamCount);
end;

function OptVal(sOption: string; sDefault: string): string;
// Returns the value from a command-line option
  var
    i: integer;
    s: string;
begin
  result := sDefault;
  for i := 1 to ParamCount do
  begin
    s := ParamStr(i);
    if CompareText(sOption+'=',copy(s,1,length(sOption+'='))) = 0 then
    begin
      result := Copy(s,pos('=',s)+1,9999);
    end;
  end;
end;

var
  ExitCode: longword;
begin
  // MAIN
  if ParamCount < 1 then
    usage;
  sHost := ParamStr(1);
  nInterval := StrToFloat(OptVal('INTERVAL','5'));
  nTimeout := StrToFloat(OptVal('TIMEOUT','1'));
  nMaxFail := StrToInt(OptVal('MAXFAIL','5'));
  sFailCmd := OptVal('FAILCMD','');

  oPinger := TPinger.Create(sHost);
  oPinger.ICMP.OnReply := oPinger.ICMPReply;

  sLogFileName := ExtractFilePath(ParamStr(0))+'pinger-'+FormatDateTime('yyyymmdd_hhnnss',Now)+'.log';
  AssignFile(LogFile, sLogFileName);
  Rewrite(LogFile);
  Log('Command: '+CmdLine);
  Log('');
  Log('Ping times (ms) to '+ParamStr(1)+':');
  Log('');
  while true do
  begin
    try
      oPinger.ICMP.Ping;
    finally
      sleep(round(nInterval * 1000));
    end;
    If (nConsecutiveFails = nMaxFail) or (nConsecutiveFails = 5*nMaxFail) or (nConsecutiveFails = 20*nMaxFail) then
    begin
      if length(sFailCmd) > 0 then
        DosCmd(sFailCmd,'',false,ExitCode);
    end;
  end;
end.